#include "panic.h"
#include "textmode.h"
#include "../lib/stdio.h"
#include "../cpu/cpu_check.h"

/**
 * Halts the system.
 */
static void stop_machine() {
    asm volatile("cli\n" "loop:\n" "hlt\n" "jmp loop");
}

/**
 * Helper function for kpanic which prints a cpu data dump for debugging
 * purposes.
 */
static void print_cpu_dump() {
    uQWord regs[8];
    cpu_get_registers(regs);
    kprintf("EAX: %#.16x\tEBX: %#.16x\nECX: %#.16x\tEDX: %#.16x\nESI: %#.16x\t"
            "EDI: %#.16x\nESP: %#.16x\tEBP: %#.16x", regs[0], regs[1], regs[2],
            regs[3], regs[4], regs[5], regs[6], regs[7]);
}

/**
 * Emits a fatal error message with debugging information and finally
 * halts the system.
 *
 * @param errcode   An error code.
 * @param message   A textual error message.
 * @param file      The offending file.
 * @param line      The offending line.
 */
void kpanic(uQWord errcode, const char* message, const char* file, int line) {
    kprintf("-= SYSTEM FAILURE: Code %#.16x =-\nMessage:\t%s\nFile:\t\t%s "
            "(line %i)\nCurrent CPU state:\n", errcode, message, file, line);
    print_cpu_dump();
    kprint("\n\nPlease restart your machine.");
    stop_machine();
}
